package liste_articles;

import java.util.ArrayList;

/**
 * Définition et gestion d'une liste d'articles.
 * 
 * @author santini
 *
 */
public class ListeArticles implements Observateur {
	
	private ArrayList<Article> listing ;
	private final String denomination ;
	
	/**
	 * Constructeur champ à champ.
	 * @param denom la dénomination de la liste.
	 * @param liste une liste d'articles prédéfinie permettant d'initialiser le contenu de la liste d'artciles.
	 */
	public ListeArticles(String denom, ArrayList<Article> liste) throws DenominationException {
		if (denom.length()<=30) {
			this.denomination = denom ;
		} else {
			throw new DenominationException("La description de la dénomination ne doit pas excéder 30 caractères",denom);
		}
		this.listing = liste;
	}
	
	/**
	 * Constructeur partiel (par de constructeur par défaut.
	 * @param denom la dénomination de la liste.
	 */
	public ListeArticles(String denom) throws DenominationException {
		this(denom, new ArrayList<Article>());
	}
	
	/**
	 * Getter : retourne l'élement de la liste enregistré à une position donnée.
	 * @param i indice de position dans la liste.
	 * @return l'article de la liste.
	 */
	public Article getArticle(Integer i) {
		return this.listing.get(i);
	}
	
	/**
	 * Ajoute un article (à la fin) de la liste d'articles.
	 * @param new_article le nouvel article ajouté à la liste.
	 */
	public void ajouterArticle(Article new_article) {
		new_article.ajouterObservateur(this);             // <-----------
		this.listing.add(new_article) ;
	}
	
	/**
	 * Supprime la première occurence d'un article de la liste.
	 * @param article article à supprimer.
	 */
	public void supprimerArticle(Article article) {
		article.supprimerObservateur(this);              // <-----------
		this.listing.remove(article) ;
	}
	
	/**
	 * Affiche la liste d'article de façon formatée.
	 */
	public void affichage() {
		System.out.println(this.toString());
	}
	
	/**
	 * Redéfinition de la méthode pour la classe ListeArticles.
	 * @return la chaîne de caractères formatée décrivant une liste d'articles.
	 */
	public String toString() {
		String s = this.denomination+" : \n";
		for (Article a : this.listing) {
			s += a.toString()+"\n";
		}
		return s;
	}
	
	/**
	 * Actualise l'affichage de la liste d'articles suite à la modification de l'état d'un article.
	 */
	public void actualiser() {
		this.affichage();
	}
}